@@ -6,4 +6,11 @@ reflekt_columns = [
     "source_schema",
     "source_table",
     "tracking_plan",
+    "event_text",
+    "call_type",
+]
+
+reflekt_reserved_columns = [
+    "user_id",
+    "anonymous_id",
 ]
