@@ -7,12 +7,11 @@ import shutil
 from pathlib import Path
 from typing import Optional, Union
 
-import pkg_resources
 import yaml
 from inflection import titleize, underscore
 from loguru import logger
-from packaging.version import Version
-from reflekt.columns import reflekt_columns
+
+from reflekt.columns import reflekt_columns, reflekt_reserved_columns
 from reflekt.config import ReflektConfig
 from reflekt.dbt import (
     dbt_column_schema,
@@ -52,10 +51,10 @@ class ReflektTransformer(object):
     def __init__(
         self,
         reflekt_plan: ReflektPlan,
-        # database: Optional[str] = None,
-        # schema: Optional[str] = None,
         dbt_package_name: Optional[str] = None,
-        pkg_version: Optional[Version] = None,
+        dbt_pkg_dir: Optional[Path] = None,
+        tmp_pkg_dir: Optional[Path] = None,
+        models_subfolder: Optional[str] = None,
     ) -> None:
         self.reflekt_plan = reflekt_plan
         self.plan_name = str.lower(self.reflekt_plan.name)
@@ -71,15 +70,9 @@ class ReflektTransformer(object):
             self.reflekt_project = ReflektProject()
             self.project_dir = self.reflekt_project.project_dir
             self.dbt_package_name = dbt_package_name
-            self.tmp_pkg_dir = (
-                self.project_dir / ".reflekt" / "tmp" / self.dbt_package_name
-            )
-            self.dbt_pkgs_dir = self.project_dir / "dbt_packages"
-            self.dbt_pkg_path = self.dbt_pkgs_dir / self.dbt_package_name
-            self.pkg_template = pkg_resources.resource_filename(
-                "reflekt", "templates/dbt/"
-            )
-            self.pkg_version = pkg_version
+            self.dbt_pkg_dir = dbt_pkg_dir
+            self.tmp_pkg_dir = tmp_pkg_dir
+            self.models_subfolder = models_subfolder
             self.db_engine = WarehouseConnection()
             self.src_prefix = self.reflekt_project.src_prefix
             self.model_prefix = self.reflekt_project.model_prefix
@@ -275,45 +268,6 @@ class ReflektTransformer(object):
         pass
 
     def build_dbt_package(self) -> None:
-        logger.info(
-            f"Building Reflekt dbt package:"
-            f"\n        cdp: {self.cdp_name}"
-            f"\n        analytics governance tool: {self.plan_type}"
-            f"\n        tracking plan: {self.plan_name}"
-            f"\n        warehouse: {self.warehouse_type}"
-            f"\n        schema: {self.schema}"
-            f"\n        schema_alias: {self.schema_alias}"
-            f"\n        dbt package name: {self.dbt_package_name}"
-            f"\n        dbt package version: {self.pkg_version}"
-            f"\n        dbt package path: {self.dbt_pkg_path}\n"
-        )
-
-        if self.tmp_pkg_dir.exists():
-            shutil.rmtree(str(self.tmp_pkg_dir))
-        shutil.copytree(self.pkg_template, str(self.tmp_pkg_dir))
-
-        # Update the version string in templated dbt_project.yml
-        with open(self.tmp_pkg_dir / "dbt_project.yml", "r") as f:
-            dbt_project_yml_str = f.read()
-
-        dbt_project_yml_str = dbt_project_yml_str.replace(
-            "0.1.0", str(self.pkg_version)  # Template always has version '0.1.0'
-        ).replace("reflekt_package_name", self.dbt_package_name)
-
-        with open(self.tmp_pkg_dir / "dbt_project.yml", "w") as f:
-            f.write(dbt_project_yml_str)
-
-        # Set dbt_pkg_name and plan_name in README.md
-        with open(self.tmp_pkg_dir / "README.md", "r") as f:
-            readme_str = f.read()
-
-        readme_str = readme_str.replace("_DBT_PKG_NAME_", self.dbt_package_name).replace(
-            "_PLAN_NAME_", self.plan_name
-        )
-
-        with open(self.tmp_pkg_dir / "README.md", "w") as f:
-            f.write(readme_str)
-
         if self.cdp_name == "rudderstack":
             return self._dbt_package_rudderstack(self.reflekt_plan)
         elif self.cdp_name == "segment":
@@ -354,27 +308,30 @@ class ReflektTransformer(object):
         else:
             screen_viewed_props = screen_viewed_event[0].properties
 
+        user_traits = [trait for trait in reflekt_plan.identify_traits]
+        group_traits = [trait for trait in reflekt_plan.group_traits]
+
         std_segment_tables = [
             {
                 "name": "identifies",
                 "description": "A table with identify() calls.",  # noqa: E501
                 "unique_key": "identify_id",
                 "cdp_cols": seg_identify_cols,
-                "plan_cols": [],
+                "plan_cols": user_traits,
             },
             {
                 "name": "users",
                 "description": "A table with the latest traits for users.",  # noqa: E501
                 "unique_key": "user_id",
                 "cdp_cols": seg_users_cols,
-                "plan_cols": [],
+                "plan_cols": user_traits,
             },
             {
                 "name": "groups",
                 "description": "A table with group() calls.",  # noqa: E501
                 "unique_key": "group_id",
                 "cdp_cols": seg_groups_cols,
-                "plan_cols": [],
+                "plan_cols": group_traits,
             },
             {
                 "name": "pages",
@@ -453,60 +410,75 @@ class ReflektTransformer(object):
             if event.name.lower() in ["page viewed", "screen viewed"]:
                 pass  # Already handled above by std_segment_tables for loop
             else:
-                db_columns, error_msg = self.db_engine.get_columns(
-                    self.schema, segment_2_snake(event.name)
-                )
-                if error_msg is not None:
-                    self.db_errors.append(error_msg)
-                else:
-                    table_name = segment_2_snake(event.name)
-                    model_name = (
-                        f"{self.model_prefix}{self.schema}__{table_name}"
-                        if self.schema_alias is None
-                        else f"{self.model_prefix}{self.schema_alias}__{table_name}"
-                    )
-                    doc_name = (
-                        f"{self.docs_prefix}{self.schema}__{table_name}"
-                        if self.schema_alias is None
-                        else f"{self.docs_prefix}{self.schema_alias}__{table_name}"
-                    )
-                    self._template_dbt_table(
-                        dbt_src=dbt_src,
-                        table_name=table_name,
-                        table_description=event.description,
-                    )
-                    self._template_dbt_model(
-                        materialized=self.materialized,
-                        unique_key="event_id",
-                        table_name=table_name,
-                        model_name=model_name,
-                        db_columns=db_columns,
-                        cdp_cols=seg_event_cols,
-                        plan_cols=event.properties,
-                    )
-                    self._template_dbt_doc(
-                        doc_name=doc_name,
-                        model_name=model_name,
-                        model_description=event.description,
-                        db_columns=db_columns,
-                        cdp_cols=seg_event_cols,
-                        plan_cols=event.properties,
+                # Get event versions
+                multi_version_event_list = [
+                    x for x in reflekt_plan.events if x.name == event.name
+                ]
+                event_versions = [event.version for event in multi_version_event_list]
+
+                # Only template latest version of event
+                if event.version == max(event_versions):
+                    db_columns, error_msg = self.db_engine.get_columns(
+                        self.schema, segment_2_snake(event.name)
                     )
+                    if error_msg is not None:
+                        self.db_errors.append(error_msg)
+                    else:
+                        table_name = segment_2_snake(event.name)
+                        model_name = (
+                            f"{self.model_prefix}{self.schema}__{table_name}"
+                            if self.schema_alias is None
+                            else f"{self.model_prefix}{self.schema_alias}__{table_name}"
+                        )
+                        doc_name = (
+                            f"{self.docs_prefix}{self.schema}__{table_name}"
+                            if self.schema_alias is None
+                            else f"{self.docs_prefix}{self.schema_alias}__{table_name}"
+                        )
+                        self._template_dbt_table(
+                            dbt_src=dbt_src,
+                            table_name=table_name,
+                            table_description=event.description,
+                        )
+                        self._template_dbt_model(
+                            materialized=self.materialized,
+                            unique_key="event_id",
+                            table_name=table_name,
+                            model_name=model_name,
+                            db_columns=db_columns,
+                            cdp_cols=seg_event_cols,
+                            plan_cols=event.properties,
+                            event_name=event.name,
+                        )
+                        self._template_dbt_doc(
+                            doc_name=doc_name,
+                            model_name=model_name,
+                            model_description=event.description,
+                            db_columns=db_columns,
+                            cdp_cols=seg_event_cols,
+                            plan_cols=event.properties,
+                        )
 
         src_name = (
             f"{self.src_prefix}{self.schema}"
             if self.schema_alias is None
             else f"{self.src_prefix}{self.schema_alias}"
         )
-        dbt_src_path = self.tmp_pkg_dir / "models" / f"{src_name}.yml"
+        dbt_src_path = (
+            self.tmp_pkg_dir / "models" / self.models_subfolder / f"{src_name}.yml"
+        )
 
         if self.docs_in_folder:
-            docs_folder_str = str(self.tmp_pkg_dir) + "/models/docs"
+            docs_folder_str = (
+                str(self.tmp_pkg_dir) + f"/models/{self.models_subfolder}/docs"
+            )
             docs_folder_path = Path(docs_folder_str)
             docs_folder_path.mkdir(exist_ok=True)
             dbt_src_path = docs_folder_path / f"{src_name}.yml"
         else:
-            dbt_src_path = self.tmp_pkg_dir / "models" / f"{src_name}.yml"
+            dbt_src_path = (
+                self.tmp_pkg_dir / "models" / self.models_subfolder / f"{src_name}.yml"
+            )
 
         with open(dbt_src_path, "w") as f:
             yaml.dump(
@@ -537,18 +509,18 @@ class ReflektTransformer(object):
 
         logger.info(
             f"Copying dbt package from temporary path "
-            f"{self.tmp_pkg_dir} to {self.dbt_pkg_path}"
+            f"{self.tmp_pkg_dir} to {self.dbt_pkg_dir}"
         )
 
-        if self.dbt_pkg_path.exists():
-            shutil.rmtree(self.dbt_pkg_path)
+        if self.dbt_pkg_dir.exists():
+            shutil.rmtree(self.dbt_pkg_dir)
 
-        shutil.copytree(self.tmp_pkg_dir, self.dbt_pkg_path)
+        shutil.copytree(self.tmp_pkg_dir, self.dbt_pkg_dir)
 
         print("")  # Terminal newline
         logger.info(
             f"[SUCCESS] Created Reflekt dbt package '{self.dbt_package_name}' "
-            f"at: {self.dbt_pkg_path}"
+            f"at: {self.dbt_pkg_dir}"
         )
         print("")  # Cleanup stdout
 
@@ -576,12 +548,15 @@ class ReflektTransformer(object):
         # cdp_cols: dict,
         # plan_cols: list,
     ) -> None:
-        print("")  # Terminal newline
-        logger.info(f"Templating table '{table_name}' in dbt source {self.schema}")
-        dbt_tbl = copy.deepcopy(dbt_table_schema)
-        dbt_tbl["name"] = table_name
-        dbt_tbl["description"] = table_description
-        dbt_src["sources"][0]["tables"].append(dbt_tbl)
+        # Check that table does not already exist in dbt source. This can happen
+        # for Segment events with multiple versions
+        if table_name not in dbt_src["sources"][0]["tables"]:
+            print("")  # Terminal newline
+            logger.info(f"Templating table '{table_name}' in dbt source {self.schema}")
+            dbt_tbl = copy.deepcopy(dbt_table_schema)
+            dbt_tbl["name"] = table_name
+            dbt_tbl["description"] = table_description
+            dbt_src["sources"][0]["tables"].append(dbt_tbl)
 
     def _template_dbt_model(
         self,
@@ -592,6 +567,7 @@ class ReflektTransformer(object):
         db_columns: list,
         cdp_cols: dict,
         plan_cols: list,
+        event_name: Optional[str] = None,
     ) -> None:
         print("")  # Terminal newline
         logger.info(
@@ -620,7 +596,10 @@ class ReflektTransformer(object):
         for column, mapped_columns in cdp_cols.items():
             if column in db_columns or column in reflekt_columns:
                 for mapped_column in mapped_columns:
-                    if mapped_column["schema_name"] is not None:
+                    if (
+                        mapped_column["source_name"] in db_columns
+                        or column in reflekt_columns
+                    ) and mapped_column["schema_name"] is not None:
                         logger.info(
                             f"    Adding column '{mapped_column['schema_name']}' to "
                             f"model SQL"
@@ -631,14 +610,27 @@ class ReflektTransformer(object):
                             .replace("__TABLE_NAME__", table_name)
                             .replace("__PLAN_NAME__", self.plan_name)
                         )
+
+                        if event_name is not None:
+                            col_sql = col_sql.replace("__EVENT_NAME__", event_name)
+
                         mdl_sql += "\n        " + col_sql + ","
 
+        # TODO - Test this logic
         for column in plan_cols:
-            logger.info(
-                f"    Adding column '{segment_2_snake(column.name)}' to model SQL"
-            )
-            col_sql = segment_2_snake(column.name)
-            mdl_sql += "\n        " + col_sql + ","
+            if (
+                segment_2_snake(column.name) in db_columns
+                and segment_2_snake(column.name) not in reflekt_reserved_columns
+            ):
+                # If the columns is named 'interval', surround in double quotes
+                column_name = (
+                    segment_2_snake(column.name)
+                    if column.name != "interval"  # Handle columns named "interval"
+                    else '"interval"'
+                )
+                logger.info(f"    Adding column '{column_name}' to model SQL")
+                col_sql = column_name
+                mdl_sql += "\n        " + col_sql + ","
 
         mdl_sql = mdl_sql[:-1]  # Remove final trailing comma
         # fmt: off
@@ -649,7 +641,9 @@ class ReflektTransformer(object):
             "select * from renamed\n"
         )
         # fmt: on
-        model_path = self.tmp_pkg_dir / "models" / f"{model_name}.sql"
+        model_path = (
+            self.tmp_pkg_dir / "models" / self.models_subfolder / f"{model_name}.sql"
+        )
 
         with open(model_path, "w") as f:
             f.write(mdl_sql)
@@ -689,7 +683,7 @@ class ReflektTransformer(object):
                 "Invalid materialized config in reflekt_project.yml. Must be "
                 "either 'view' or 'incremental'. See Reflekt docs on project "
                 "configuration:"
-                "    https://github.com/GClunies/reflekt/blob/main/docs/DOCUMENTATION.md#reflekt-project"  # noqa: E501
+                "    https://www.notion.so/reflekt-ci/Reflekt-Project-Configuration-96d375edb06743a8b1699f480b3a2c74#68ffa7415eef443c9a6ba99c31c2d590"  # noqa: E501
             )
 
         return model_config
@@ -750,26 +744,45 @@ class ReflektTransformer(object):
                         mdl_col = copy.deepcopy(dbt_column_schema)
                         mdl_col["name"] = mapped_column["schema_name"]
                         mdl_col["description"] = mapped_column["description"]
-                        if mapped_column.get("tests") is not None:
-                            mdl_col["tests"] = mapped_column["tests"]
+                        # if mapped_column.get("tests") is not None:  # remove id tests for now  # noqa: E501
+                        #     mdl_col["tests"] = mapped_column["tests"]
                         dbt_mdl_doc["columns"].append(mdl_col)
 
         for column in plan_cols:
-            logger.info(f"    Adding column '{segment_2_snake(column.name)}' to docs")
-            mdl_col = copy.deepcopy(dbt_column_schema)
-            mdl_col["name"] = segment_2_snake(column.name)
-            mdl_col["description"] = column.description
-            dbt_mdl_doc["columns"].append(mdl_col)
+            if (
+                segment_2_snake(column.name) in db_columns
+                and segment_2_snake(column.name) not in reflekt_reserved_columns
+            ):
+                logger.info(
+                    f"    Adding column '{segment_2_snake(column.name)}' to docs"
+                )
+                mdl_col = copy.deepcopy(dbt_column_schema)
+                mdl_col["name"] = segment_2_snake(column.name)
+                mdl_col["description"] = column.description
+                dbt_mdl_doc["columns"].append(mdl_col)
+        # for column in plan_cols:
+        #     if column.name in db_columns:
+        # logger.info(
+        #     f"    Adding column '{segment_2_snake(column.name)}' to docs"
+        # )
+        # mdl_col = copy.deepcopy(dbt_column_schema)
+        # mdl_col["name"] = segment_2_snake(column.name)
+        # mdl_col["description"] = column.description
+        # dbt_mdl_doc["columns"].append(mdl_col)
 
         dbt_doc["models"].append(dbt_mdl_doc)
 
         if self.docs_in_folder:
-            docs_folder_str = str(self.tmp_pkg_dir) + "/models/docs"
+            docs_folder_str = (
+                str(self.tmp_pkg_dir) + f"/models/{self.models_subfolder}/docs"
+            )
             docs_folder_path = Path(docs_folder_str)
             docs_folder_path.mkdir(exist_ok=True)
             docs_path = docs_folder_path / f"{doc_name}.yml"
         else:
-            docs_path = self.tmp_pkg_dir / "models" / f"{doc_name}.yml"
+            docs_path = (
+                self.tmp_pkg_dir / "models" / self.models_subfolder / f"{doc_name}.yml"
+            )
 
         with open(docs_path, "w") as f:
             yaml.dump(
